#ifndef ContourTracker_H
#define ContourTracker_H

#include "Curve2D.h"
#include "Point.h"

/*!
 * \file ContourTracker.h
 * \brief  suiveur de contour implicite
 * \author Tristan Roussillon
 *
 */

/*!
 * \class ContourTracker
 * \brief  suiveur de contour implicite
 * \author Tristan Roussillon
 *
 */
class ContourTracker {

private : 

	Point<int> P;/*!< point courant objet */
	Point<int> Q;/*!< point courant fond */
	Point<int> R;/*!< point  */
	Point<int> S;/*!< point  */
//PQRS forment un carr unit orient dans le sens des aiguilles d'une montre
	Vecteur<int> v; /*!< vecteur deplacement courant */
	Curve2D* curve; /*!< pointeur vers une courbe */

	int c; /*!< compteur des points traites */

public : 
/*!
 * \brief  constructeur du suiveur de contour
 *
 * \param p : point de depart
 * \param c : une courbe a discretiser
 *
 */
	ContourTracker(Point<int> p, Curve2D* c);



/*!
 * \brief  accesseur de la lettre codant
 * le dernier deplacement
 *
 */
	std::string getLetter() const;

/*!
 * \brief Methode permettant
 * d'acceder au point suivant du contour
 */
	void operator++();
/*!
 * \brief Methode permettant
 * d'acceder au point precedant du contour
 */
	void operator--();

/*!
 * \brief operateur d'egalite
 */ 
	bool operator==(const ContourTracker& t) const;
/*!
 * \brief operateur de difference
 */ 
	bool operator!=(const ContourTracker& t) const; 
};

      
#endif
