#include "../inc/Ellipse.h"

Ellipse::Ellipse(double pcx, double pcy, double pa, double pb, double palpha) {
	//param
	this->cx = pcx;
	this->cy = pcy;
	this->a = std::abs(pa);
	this->b = std::abs(pb);
	this->alpha = palpha;
	//premier point
	//calcul de la plus petite abscisse entiere des points interieurs
/*equation parametrique d'une ellipse
 * x = a.cos(alpha).cos(t) + b.sin(alpha).sin(t)
 * y = -a.sin(alpha).cos(t) + b.cos(alpha).sin(t)
 * on cherche x, minimum, donc
 * t min a.cos(alpha).cos(t) + b.sin(alpha).sin(t)
 * derivee -a.cos(alpha).sin(t) + b.sin(alpha).cos(t) = 0
 * tan(t) = (b/a).tan(alpha)
*/

	double t = std::atan((this->b/this->a)*std::tan(alpha));
	if (t < 0) t = t + M_PI;
	double t2 = t + M_PI;

	double x = std::min(this->a*std::cos(this->alpha)*std::cos(t) + this->b*std::sin(this->alpha)*std::sin(t), this->a*std::cos(this->alpha)*std::cos(t2) + this->b*std::sin(this->alpha)*std::sin(t2))+this->cx;
	int xmin = (int)std::ceil(x); 

	//calcul du point de plus grand ordonnee d'abscisse xmin
/* trouver t | x = a.cos(alpha).cos(t) + b.sin(alpha).sin(t) = xmin 
 * puis utiliser y = -a.sin(alpha).cos(t) + b.cos(alpha).sin(t) 
*/
	double A = this->a*std::cos(this->alpha);
	double B = this->b*std::sin(this->alpha);
	double module = std::sqrt(A*A+B*B);


	double arg = std::atan2(B,A);               
	double t3 = std::acos((xmin-this->cx)/module) + arg;
	double t4 = -std::acos((xmin-this->cx)/module) + arg;

              
	double y = std::max(-this->a*std::sin(this->alpha)*std::cos(t3) + this->b*std::cos(this->alpha)*std::sin(t3),-this->a*std::sin(this->alpha)*std::cos(t4) + this->b*std::cos(this->alpha)*std::sin(t4))+this->cy;
	int ymax = (int)std::floor(y); 
//
	this->firstPoint = Point<int>(xmin,ymax);
	//dernier point
	this->lastPoint = this->firstPoint;
}


bool Ellipse::isInside(const Point<int>& p) const {
	//var locales
	double d, dcalc, theta;
	double e2 = (this->b*this->b - this->a*this->a)/(this->b*this->b);
	//distance au centre
	d = (p.x-this->cx)*(p.x-this->cx) + (p.y-this->cy)*(p.y-this->cy);
	//distance au centre calcule
	theta = std::atan2((p.x-this->cx),(p.y-this->cy)) - alpha;  
	dcalc = (this->a*this->a)/(1-e2*(std::cos(theta)*std::cos(theta)));
	return (d <= dcalc);
}

