#ifndef AbstractContourIterator_H
#define AbstractContourIterator_H

#include "Contour.h"

/*!
 * \file AbstractContourIterator.h
 * \brief  definition d'un iterateur sur un contour
 * \author Tristan Roussillon
 *
 */

/*!
 * \class AbstractContourIterator
 * \brief iterateur sur un contour
 * \author Tristan Roussillon
 *
 */
class AbstractContourIterator {

public : 

	Contour* c; /*!< pointeur vers un contour */

	int i; /*!< indice du point courant */

public : 

/*!
 * \brief Destructeur virtuel
 */
	virtual ~AbstractContourIterator() {};

/*!
 * \brief Methode permettant
 * d'acceder au point du contour
 */
	Point<int> operator*();

/*!
 * \brief Methode de copie
 */
	virtual AbstractContourIterator* copy() = 0;

/*!
 * \brief Methode permettant
 * d'avancer l'iterateur
 */
	virtual void operator++() = 0;
/*!
 * \brief Methode permettant
 * de reculer l'iterateur
 */
	virtual void operator--() = 0;

/*!
 * \brief operateur d'egalite
 */ 
	virtual bool operator==(const AbstractContourIterator& t) const;
/*!
 * \brief operateur de difference
 */ 
	virtual bool operator!=(const AbstractContourIterator& t) const; 
};

      
#endif
