#ifndef AbstractSegIterator_H
#define AbstractSegIterator_H

#include "ContourIterator.h"
#include "FactorySeg.h"
#include "Seg.h"

/*!
 * \file AbstractSegIterator.h
 * \brief definition d'un iterateur sur un segment
 * \author Tristan Roussillon
 *
 */

/*!
 * \class AbstractSegIterator
 * \brief iterateur sur un segment
 * \author Tristan Roussillon
 *
 */
class AbstractSegIterator {

protected :

ContourIterator i; /*!< debut du seg */
ContourIterator j; /*!< fin du seg */

public : 

	AbstractSegIterator(
									const ContourIterator& pi, 
									const ContourIterator& pj);

/*!
 * \brief Destructeur virtuel
 */
	virtual ~AbstractSegIterator() {}

/*!
 * \brief Methode permettant
 * d'acceder au segment
 */
	virtual Seg* operator*() = 0;
/*!
 * \brief Methode permettant
 * d'acceder au segment
 */
	virtual Seg* getSeg() = 0;

/*!
 * \brief Methode permettant
 * d'acceder a un iterateur
 * pointant sur le point
 * du contour suivant le dernier
 * point du segment
 */
	ContourIterator getFront();

/*!
 * \brief Methode permettant
 * d'acceder a un iterateur
 * pointant sur le premier 
 * point du segment
 */
	ContourIterator getBack();

/*!
 * \brief Methode permettant
 * d'avancer l'iterateur
 */
	virtual void operator++() = 0;

/*!
 * \brief operateur d'egalite
 */ 
	virtual bool operator==(AbstractSegIterator& t) = 0;
/*!
 * \brief operateur de difference
 */ 
	virtual bool operator!=(AbstractSegIterator& t) = 0; 
};

      
#endif
