#ifndef AdapterContainer_H
#define AdapterContainer_H
/*!
 * \file AdapterContainer.h
 * \brief Definition d'un adaptateur a un conteneur
 * \author Tristan Roussillon
 *
 */


/*!
 * \class AdapterContainer
 * \brief Classe implementant le design pattern
 * de l'adaptateur (adapter) pour associer 
 * la bonne extremite du conteneur
 * a l'extremite etiquette fin
 * \author Tristan Roussillon
 * 
*/
//T type de l'elt, Tc type du conteneur
template <class T,class Tc> class AdapterContainer {
protected : 
	Tc *pd; /*!< pointeur vers le conteneur */
public : 
/*!
 * \brief Methode virtuelle pour 
 * supprimer un point a une extremite
 */
	virtual void pop() const = 0;
/*!
 * \brief Methode virtuelle pour 
 * ajouter un point a une extremite
 * \param p : un point 
 */
	virtual void push(const T& p) const = 0;
/*!
 * \brief Methode virtuelle pour 
 * acceder au point a une extremite
 * du conteneur 
 */
	virtual T end() const = 0;

	virtual ~AdapterContainer() {}
};

#endif

