#ifndef AdapterContainerBack_H
#define AdapterContainerBack_H

/*!
 * \file AdapterContainerBack.h
 * \brief Definition d'un adaptateur a un conteneur
 * \author Tristan Roussillon
 *
 */


/*!
 * \class AdapterContainerBack
 * \brief Classe implementant le design pattern
 * de l'adaptateur (adapter) pour associer 
 * l'extremite arriere du conteneur
 * a l'extremite etiquette fin
 * \author Tristan Roussillon
 * 
*/

template <class T,class Tc> 
class AdapterContainerBack : public AdapterContainer<T,Tc> {
public : 
	AdapterContainerBack(Tc*);
/*!
 * \brief Supprimer un point a l'arriere
 */
	void pop() const;
/*!
 * \brief Ajouter un point a l'arriere
 */
	void push(const T&) const;
/*!
 * \brief Acceder a l'arriere
 */
	T end() const;
};


template <class T,class Tc> 
AdapterContainerBack<T,Tc>::AdapterContainerBack(Tc* deque) {
	this->pd = deque;
}

template <class T,class Tc> 
void AdapterContainerBack<T,Tc>::pop() const {
	this->pd->pop_back();
}
template <class T,class Tc> 
void AdapterContainerBack<T,Tc>::push(const T& p) const {
	this->pd->push_back(p);
}
template <class T,class Tc> 
T AdapterContainerBack<T,Tc>::end() const {
	return this->pd->back();
}

#endif

