#ifndef AdapterContainerFront_H
#define AdapterContainerFront_H

/*!
 * \file AdapterContainerFront.h
 * \brief Definition d'un adaptateur a un conteneur
 * \author Tristan Roussillon
 *
 */


/*!
 * \class AdapterContainerFront
 * \brief Classe implementant le design pattern
 * de l'adaptateur (adapter) pour associer 
 * l'extremite avant du conteneur
 * a l'extremite etiquette fin
 * \author Tristan Roussillon
 * 
*/

template <class T,class Tc> class 
AdapterContainerFront : public AdapterContainer<T,Tc> {
public : 
	AdapterContainerFront(Tc*);
/*!
 * \brief Supprimer un point a l'avant
 */
	void pop() const;
/*!
 * \brief Ajouter un point a l'avant
 */
	void push(const T&) const;
/*!
 * \brief Acceder a l'avant
 */
	T end() const;
};

template <class T,class Tc> 
AdapterContainerFront<T,Tc>::AdapterContainerFront(Tc* deque) {
	this->pd = deque;
}

template <class T,class Tc> 
void AdapterContainerFront<T,Tc>::pop() const {
	this->pd->pop_front();
}
template <class T,class Tc> 
void AdapterContainerFront<T,Tc>::push(const T& p) const {
	this->pd->push_front(p);
}
template <class T,class Tc> 
T AdapterContainerFront<T,Tc>::end() const {
	return this->pd->front();
}
#endif
