#ifndef BackFrontCover_H
#define BackFrontCover_H

#include "Contour.h"
#include "ContourIterator.h"
#include "ContourScan.h"


#include "FactoryFrontBackSeg.h"
#include "BackFrontSegIterator.h"
#include "SegIterator.h"


/*!
 * \file BackFrontCover.h
 * \brief Couverture de segments
 * \author Tristan Roussillon
 *
 */

/*!
 * \class BackFrontCover
 * \brief Couverture de segments
 * \author Tristan Roussillon
 *
 */
class BackFrontCover {

private : 
	std::vector<FrontBackSeg*> vSegs; /*!< ensemble des segments extraits au cours des iterations*/
	std::vector<FrontBackSeg*> vTmp; /*!< ensemble des segments temporaires */
	AbstractSegIterator* firstSeg; /*!< premier segment du contour */
	AbstractSegIterator* lastSeg; /*!< dernier segment du contour */

public : 
/*!
 * \brief  constructeur 
 *
 * \param pc : un pointeur vers un contour
 *
 */
	BackFrontCover(Contour* pc, FactoryFrontBackSeg* pf);
/*!
 * \brief  destructeur
 *
 */
	~BackFrontCover();

/*!
 * \brief  pointeur vers le premier point du contour
 *
 */
	SegIterator begin() const;
/*!
 * \brief  pointeur vers le dernier point du contour
 *
 */
	SegIterator end() const;

};

      
#endif
