#ifndef BackFrontSegIterator_H
#define BackFrontSegIterator_H

#include "AbstractSegIterator.h"
#include "ContourIterator.h"
#include "FactoryFrontBackSeg.h"

/*!
 * \file BackFrontSegIterator.h
 * \brief  definition d'un iterateur de segments maximaux avec ajout a l'avant et a l'arriere
 * \author Tristan Roussillon
 *
 */

/*!
 * \class BackFrontSegIterator
 * \brief iterateur de segments maximaux sur un contour avec ajout a l'avant et a l'arriere
 * \author Tristan Roussillon
 *
 */
class BackFrontSegIterator: public AbstractSegIterator {

protected : 

FactoryFrontBackSeg* fseg; /*!< fabrique du seg */
std::vector<FrontBackSeg*>* v; /*!< pointeur vers un ensemble de segments */

public : 

/*!
 * \brief Constructeur
 * \param pf : fabrique du segment
 * \param pv : ensemble de segments
 * \param pi : iterateur de contour sur le premier point du segment
 * \param pj : iterateur de contour sur le dernier point du segment
 */
 BackFrontSegIterator(FactoryFrontBackSeg* pf, 
											std::vector<FrontBackSeg*>* pv, 
											const ContourIterator& pi, 
											const ContourIterator& pj);

/*!
 * \brief Destructeur
 */
	virtual ~BackFrontSegIterator();
/*!
 * \brief Methode permettant
 * d'acceder au segment
 */
	Seg* operator*();
/*!
 * \brief Methode permettant
 * d'acceder au segment
 */
	Seg* getSeg();
/*!
 * \brief Methode permettant
 * d'avancer l'iterateur
 */
	virtual void operator++() = 0;

};

      
#endif
