#ifndef BruteForceSegIterator_H
#define BruteForceSegIterator_H

#include "AbstractSegIterator.h"
#include "ContourIterator.h"

/*!
 * \file BruteForceSegIterator.h
 * \brief  definition d'un iterateur naif de segments maximaux 
 * \author Tristan Roussillon
 *
 */

/*!
 * \class BruteForceSegIterator
 * \brief iterateur naif de segments maximaux sur un contour
 * \author Tristan Roussillon
 *
 */
class BruteForceSegIterator: public AbstractSegIterator {

protected : 

FactorySeg* fseg; /*!< fabrique d'un segment */
std::vector<Seg*>* v; /*!< pointeur vers un ensemble de segments */

public : 

/*!
 * \brief Constructeur
 * \param pf : fabrique du segment
 * \param pv : ensemble de segments
 * \param pi : iterateur de contour sur le premier point du segment
 * \param pj : iterateur de contour sur le dernier point du segment
 */
 BruteForceSegIterator(FactorySeg* pf, 
											std::vector<Seg*>* pv, 
											const ContourIterator& pi, 
											const ContourIterator& pj);

/*!
 * \brief Destructeur
 */
	virtual ~BruteForceSegIterator();
/*!
 * \brief Methode permettant
 * d'acceder au segment
 */
	Seg* operator*();
/*!
 * \brief Methode permettant
 * d'acceder au segment
 */
	Seg* getSeg();
/*!
 * \brief Methode permettant
 * d'avancer l'iterateur
 */
	virtual void operator++() = 0;

};

      
#endif
