#ifndef Contour_H
#define Contour_H

#include "Point.h"
#include "BadConnectivity.h"
#include <vector>
#include <string>
#include <sstream>

/*!
 * \file Contour.h
 * \brief definition d'un contour
 * \author Tristan Roussillon
 *
 */


/*!
 * \class Contour
 * \brief Contour (chemin 4-connexe simple)
 * \author Tristan Roussillon
 */

class Contour {

private : 

bool open; /*!< booleen indiquant si le contour est incomplet */

std::vector<Point<int> > vPts; /*!< vecteur des points du contour */


/*!
 * \brief methode renvoyant le vecteur associe a un caractere du code
 * \param un caractere du code (0,1,2 ou 3)
 *
 */

Vecteur<int> letter2Vec(const char&);

 
public : 

/*!
 * \brief constructeur
 * \param une chaine de caractere contenant le point de depart et le code de freeman
 *
 */
 	Contour(std::string s);

/*!
 * \brief constructeur
 * \param point de depart
 * \param code de freeman
 *
 */
 	Contour(Point<int> pt, std::string code);

/*!
 * \brief constructeur
 * \param suite de points
 *
 */
 	Contour(std::vector<Point<int> > v);

/*!
 * \brief destructeur
 */
	virtual ~Contour() {}

/*!
 * \brief predicat renvoyant TRUE si le contour est incomplet, FALSE sinon
 *
 */
 	bool isOpen() const;

/*!
 * \brief accesseur a un point du contour
 *
 */
 	Point<int> at(int i) const;

/*!
 * \brief accesseur a la longueur du contour
 *
 */
 	int getLength() const;

/*!
 * \brief accesseur a l'ensemble des points du contour
 *
 */
 	std::vector<Point<int> > getPts() const;

};

      
#endif

