#ifndef ContourIterator_H
#define ContourIterator_H

#include "AbstractContourIterator.h"

/*!
 * \file ContourIterator.h
 * \brief  definition d'un iterateur sur un contour
 * \author Tristan Roussillon
 *
 */

/*!
 * \class ContourIterator
 * \brief iterateur sur un contour
 * \author Tristan Roussillon
 *
 */
class ContourIterator {

private : 

	AbstractContourIterator* c; /*!< pointeur vers un iterateur */

public : 

/*!
 * \brief Constructeur 
 * \param pc : pointeur vers un iterateur de contour
 */
	ContourIterator(AbstractContourIterator* pc); 
/*!
 * \brief Constructeur de copie
 * \param un iterateur de contour
 */
	ContourIterator(const ContourIterator&);
/*!
 * \brief operateur d'affectation
 * \param un iterateur de contour
 */
	ContourIterator& operator=(const ContourIterator&);

/*!
 * \brief Destructeur 
 */
	~ContourIterator();

/*!
 * \brief Methode permettant
 * d'acceder au point du contour
 */
	Point<int> operator*();

/*!
 * \brief Methode permettant
 * d'avancer l'iterateur
 */
	void operator++();
/*!
 * \brief Methode permettant
 * de reculer l'iterateur
 */
	void operator--();

/*!
 * \brief operateur d'egalite
 */ 
	bool operator==(const ContourIterator& t) const;
/*!
 * \brief operateur de difference
 */ 
	bool operator!=(const ContourIterator& t) const; 
};

      
#endif
