#ifndef DSS8_H
#define DSS8_H

#include "DSS.h"
/*!
 * \file DSS8.h
 * \brief Definition d'une classe modelisant un segment de droite 8-connexe
 * selon sa caracterisation arithmetique (reveilles91, debled95)
 * \author Tristan Roussillon
 *
 */


/*!
 * \class DSS8
 * \brief segment de droite arithmetique 8-connexe
 * \author Tristan Roussillon
 * cette classe modelise un segment de droite arithmetique 8-connexe.
 */
class DSS8 : public DSS {


public:
	DSS8(const Point<int>&, const Point<int>&); //constructeur
/*!
 * \brief Donne l'epaisseur du segment,
 * max(a,b)
 */
	int getOmega() const;


private:

/*!
 * \brief Methode
 * retournant le vecteur deplacement
 * permettant de trouver le point de Bezout
 * (point n'appartenant pas au segment le plus proche du segment)
 *  a partir du point d'appui inferieur. 
 * Dans le cas 8-connexe, c'est le vecteur (0, +-1) ou (+-1, 0).
 */ 
	Vecteur<int> shift() const;
/*!
 * \brief Methode 
 * retournant le point suivant le
 * premier point du segment. 
 * \param pfirst : premier point du segment
 * \param plast : dernier point du segment
 *
 */
	Point<int> nextPt(const Point<int>& pfirst, const Point<int>& plast) const;
/*!
 * \brief Methode virtuelle pure, 
 * retournant TRUE si P et Q sont 8-voisins, 
 * FALSE sinon.
 * \param P : un point
 * \param Q : un point 
 *
 */
	bool areNeighbors(const Point<int>& P, const Point<int>& Q) const;
/*!
 * \brief Methode virtuelle pure, 
 * retournant TRUE si N est strictement 8-voisin de plast, 
 * FALSE sinon.
 * \param pfirst : premier point du segment
 * \param plast : dernier point du segment
 * \param N : le point ajoute
 *
 */
	bool isBadConnected(const Point<int>& pfirst, const Point<int>& plast, const Point<int>& N) const;

};
#endif


