#ifndef FP_H
#define FP_H

#include "Contour.h"
#include "ContourScan.h"
#include "ContourIterator.h"
#include "DSS4.h"
#include "AdapterDSS1.h"
#include "AdapterDSS0.h"

#include <vector>
#include <list>

/*!
 * \file FP.h
 * \brief Definition d'une classe calculant
 * le polygone reversible et respectant les 
 * parties convexes et concaves du contour
 * en utilisant les methodes
 * push_front et pop_back de la classe DSS4
 * \author Tristan Roussillon
 *
 */


/*!
 * \class FP
 * \brief Classe calculant  
 * le polygone reversible et respectant les 
 * parties convexes et concaves du contour
 * en utilisant les methodes
 * push_front et pop_back de la classe DSS4
* \author Tristan Roussillon
 * 
*/
class FP {


private :

	std::list<Point<int> > vPts; /*!< ensemble des points du polygone */

public : 
/*!
 * \brief Constructeur
 * \param pc : un contour
 * (le polygone est calcule
 * a partir du contour)
 */
	FP(Contour* pc); 
 
/*!
 * \brief Constructeur
 * \param pv : une sequence de sommets
 */
	FP(const std::vector<Point<int> >& pv); 
 
/*!
 * \brief Accede au polygone
 */
	std::vector<Point<int> > getFP() const;

/*!
 * \brief Retourne le MLP
 */
	std::vector<Point<double> > getMLP() const;

/*!
 * \brief Retourne le contour a partir duquel il a ete calcule
 */
	Contour getContour() const;

};

//fonctions outils

bool quadrant(Vecteur<int>, int);

Vecteur<double> getShift(Point<int>, Point<int>, Point<int>);

std::vector<Point<int> > draw(int,int,int,int,int,Point<int>,Point<int>);

std::vector<Point<int> > getEdge(Point<int>, Point<int>, Point<int>, Point<int>);

std::vector<Point<int> > getUpperEdge(Point<int>, Point<int>);
std::vector<Point<int> > getLowerEdge(Point<int>, Point<int>);

#endif

