#ifndef FactoryDSS_H
#define FactoryDSS_H

#include "FactoryPushPopSeg.h"
#include "DSS.h"
#include "AdapterDSS.h"
/*!
 * \file FactoryDSS.h
 * \brief Definition d'une fabrique a segments de droite (arithmetiques)
 * \author Tristan Roussillon
 *
 */


/*!
 * \class FactoryDSS
 * \brief fabrique a segments de droite (arithmetiques)
 * \author Tristan Roussillon
 * cette classe, abstraite, modelise une fabrique a segments de droite (arithmetiques)
 * selon le modele de conception de la fabrique abstraite (abstract factory).
 */
class FactoryDSS : public FactoryPushPopSeg {
public : 
/*!
 * \brief Methode virtuelle pure, 
 * permettant de creer un segment de droite
 * 
 * \param point
 * \param point
 */
	virtual DSS* createSeg(const Point<int>&, const Point<int>&) const = 0;
/*!
 * \brief Methode virtuelle pure, 
 * permettant de creer un segment de droite
 * 
 * \param un pointeur vers un segment de droite (DSS)
 */
	virtual DSS* createSeg(PushPopSeg*) const = 0;
/*!
 * \brief Methode 
 * permettant de creer un adaptateur
 * a un segment de droite
 * 
 * \param un pointeur vers un segment de droite (DSS)
 * \param un bouleen (donnant l'orientation)
 */
	AdapterDSS* createAdapterDSS(DSS*, bool);

	virtual ~FactoryDSS() {}
};

#endif

