#ifndef GreedySegIterator_H
#define GreedySegIterator_H

#include "AbstractSegIterator.h"
#include "ContourIterator.h"

/*!
 * \file GreedySegIterator.h
 * \brief  definition d'un iterateur glouton de segments 
 * \author Tristan Roussillon
 *
 */

/*!
 * \class GreedySegIterator
 * \brief iterateur glouton de segments sur un contour
 * \author Tristan Roussillon
 *
 */
class GreedySegIterator: public AbstractSegIterator {

private : 

ContourIterator end; /*!< fin du contour */

FactorySeg* fseg; /*!< fabrique du seg */
std::vector<Seg*>* v; /*!< pointeur vers un ensemble de segments */

public : 

/*!
 * \brief Constructeur
 * \param pf : fabrique du segment
 * \param pv : ensemble de segments
 * \param pi : iterateur de contour sur le premier point du segment
 * \param pj : iterateur de contour sur le dernier point du segment
 * \param pend : iterateur de contour sur la fin du contour
 */
 GreedySegIterator(FactorySeg* pf, 
									std::vector<Seg*>* pv, 
									const ContourIterator& pi, 
									const ContourIterator& pj, 
									const ContourIterator& pend);

/*!
 * \brief Destructeur
 */
	~GreedySegIterator();
/*!
 * \brief Methode permettant
 * d'acceder au segment
 */
	Seg* operator*();
/*!
 * \brief Methode permettant
 * d'acceder au segment
 */
	Seg* getSeg();
/*!
 * \brief Methode permettant
 * d'avancer l'iterateur
 */
	void operator++();

/*!
 * \brief operateur d'egalite
 */ 
	bool operator==(AbstractSegIterator& t);
/*!
 * \brief operateur de difference
 */ 
	bool operator!=(AbstractSegIterator& t); 

};

      
#endif
