#ifndef Predicat_H
#define Predicat_H

#include "Inequality.h"
#include "../src_obj/InequalityL.cpp"
#include "../src_obj/InequalityG.cpp"
#include "../src_obj/InequalityLEQ.cpp"
#include "../src_obj/InequalityGEQ.cpp"

//#define DEBUG_PRED

/*!
 * \file Predicat.h
 * \brief Predicat d'orientation
 * \author Tristan Roussillon
 *
 */


/*!
 * \class Predicat
 * \brief Predicat d'orientation
 * \author Tristan Roussillon
 * 
*/
template <class T> class Predicat {

protected :
	Inequality<T> *ineq; /*! pointeur vers une inegalite (<, >, <=, >=) */
	
/*!
 * \brief Calcul du predicat 
 * \param 3 points
 * \param une inegalite
 */
	virtual bool test(const Point<T>&, const Point<T>&, const Point<T>&, Inequality<T> *i);

public :

/*!
 * \brief Constructeur
 */
	Predicat(); 
/*!
 * \brief Constructeur
 * \param isLeft (booleen) 
 */
	Predicat(bool isLeft); 
/*!
 * \brief Constructeur
 * \param isLeft (booleen) 
 * \param isStrict (booleen)
 */
	Predicat(bool isLeft, bool isSrict); 

/*!
 * \brief Destructeur
 */
	virtual ~Predicat();

/*!
 * \brief Modification
 */
	virtual void Modify(bool isLeft, bool isSrict); 
/*!
 * \brief Inverse le signe
 */
	virtual void ReverseSign();
/*!
 * \brief Inverse l'egalite
 */
	virtual void ReverseEq();

//on ne met pas virtual pour le clonage
//a cause du type de retour
/*!
 * \brief Clonage
 */
	Predicat<T> Copy(bool isLeft, bool isSrict) const; 
/*!
 * \brief Clonage (renvoi un pointeur)
 */
	Predicat<T>* CopyPtr(bool isLeft, bool isSrict) const; 

/*!
 * \brief obtenir l'air signee du triangle pqr
 * \param p : point
 * \param q : point
 * \param r : point
 */
	T getArea(const Point<T>& p, const Point<T>& q, const Point<T>& r); 
/*!
 * \brief renvoie vrai quand le predicat d'orientation est vrai
 * \param p : point
 * \param q : point
 * \param r : point
 */
	virtual bool isGood(const Point<T>&, const Point<T>&, const Point<T>&);
/*!
 * \brief renvoie vrai quand le predicat d'orientation est faux
 * \param p : point
 * \param q : point
 * \param r : point
 */
	virtual bool isNoGood(const Point<T>&, const Point<T>&, const Point<T>&);
};

//constructeurs 
template <class T> Predicat<T>::Predicat() {
	this->ineq = new InequalityL<T>();
}

template <class T> Predicat<T>::Predicat(bool isLeft) {
	if (isLeft) this->ineq = new InequalityL<T>();
	else this->ineq = new InequalityG<T>();
}

template <class T> Predicat<T>::Predicat(bool isLeft, bool isStrict) {
	if (isStrict) {
		if (isLeft) this->ineq = new InequalityLEQ<T>();
		else this->ineq = new InequalityGEQ<T>();
	} else {
		if (isLeft) this->ineq = new InequalityL<T>();
		else this->ineq = new InequalityG<T>();
	}
}

//destructeur
template <class T> Predicat<T>::~Predicat() {
	delete this->ineq;
}

//modification
template <class T> 
void Predicat<T>::Modify(bool isLeft, bool isStrict) {
	Inequality<T> *tmp;
	if (isStrict) {
		if (isLeft) tmp = new InequalityLEQ<T>();
		else tmp = new InequalityGEQ<T>();
	} else {
		if (isLeft) tmp = new InequalityL<T>();
		else tmp = new InequalityG<T>();
	}
	delete this->ineq;
	this->ineq = tmp;
} 
template <class T> 
void Predicat<T>::ReverseSign() {
	Inequality<T> *tmp;
	std::string sign = this->ineq->toString();
	if (sign == "<=") tmp = new InequalityGEQ<T>();
	else if (sign == "<") tmp = new InequalityG<T>();
	else if (sign == ">=") tmp = new InequalityLEQ<T>();
	else if (sign == ">") tmp = new InequalityL<T>();
	delete this->ineq;
	this->ineq = tmp;
}
template <class T> 
void Predicat<T>::ReverseEq() {
	Inequality<T> *tmp;
	std::string sign = this->ineq->toString();
	if (sign == "<=") tmp = new InequalityL<T>();
	else if (sign == "<") tmp = new InequalityLEQ<T>();
	else if (sign == ">=") tmp = new InequalityG<T>();
	else if (sign == ">") tmp = new InequalityGEQ<T>();
	delete this->ineq;
	this->ineq = tmp;
}

//clonage
template <class T> 
Predicat<T> Predicat<T>::Copy(bool isLeft, bool isStrict) const {
	if (isStrict) {
		if (isLeft) return Predicat<T>(true, true);
		else return Predicat<T>(false, true);
	} else {
		if (isLeft) return Predicat<T>(true, false);
		else return Predicat<T>(false, false);
	}
} 
 
template <class T> 
Predicat<T>* Predicat<T>::CopyPtr(bool isLeft, bool isStrict) const {
	if (isStrict) {
		if (isLeft) return new Predicat<T>(true, true);
		else return new Predicat<T>(false, true);
	} else {
		if (isLeft) return new Predicat<T>(true, false);
		else return new Predicat<T>(false, false);
	}
} 

template <class T> T Predicat<T>::getArea(const Point<T>& p, const Point<T>& q, const Point<T>& r) {
	//calcul des vecteurs pq et pr
	Vecteur<T> pq = q - p;
	Vecteur<T> pr = r - p;
	//calcul de det(pq, pr) (= aire du parallelogramme)
	T det = ((pq.x * pr.y) - (pq.y * pr.x));
	//std::cout << det << std::endl;
	return det;
}

template <class T> 
bool Predicat<T>::test(const Point<T>& p, const Point<T>& q, const Point<T>& r, Inequality<T> *i) {

	T area = this->getArea(p,q,r);
	T leftpart = area;
	T rightpart = 0;
	#ifdef DEBUG_PRED
		std::cout << leftpart << " " << i->toString() << " " << rightpart << std::endl;
	#endif
	return i->test(leftpart,rightpart);
}
template <class T> bool Predicat<T>::isNoGood(const Point<T>& p, const Point<T>& q, const Point<T>& r) {
	return this->test(p,q,r, this->ineq);
}

template <class T> bool Predicat<T>::isGood(const Point<T>& p, const Point<T>& q, const Point<T>& r) {
	return (!this->test(p,q,r, this->ineq));
}
   #endif

