#ifndef PushPopCover_H
#define PushPopCover_H

#include "Contour.h"
#include "ContourIterator.h"
#include "ContourScan.h"


#include "FactoryPushPopSeg.h"
#include "PushPopSegIterator.h"
#include "SegIterator.h"


/*!
 * \file PushPopCover.h
 * \brief Couverture de segments
 * \author Tristan Roussillon
 *
 */

/*!
 * \class PushPopCover
 * \brief Couverture de segments
 * \author Tristan Roussillon
 *
 */
class PushPopCover {

private : 
	std::vector<PushPopSeg*> vSegs; /*!< ensemble des segments extraits au cours des iterations*/
	std::vector<PushPopSeg*> vTmp; /*!< ensemble des segments temporaires */
	AbstractSegIterator* firstSeg; /*!< premier segment du contour */
	AbstractSegIterator* lastSeg; /*!< dernier segment du contour */

public : 
/*!
 * \brief  constructeur 
 *
 * \param pc : un pointeur vers un contour
 *
 */
	PushPopCover(Contour* pc, FactoryPushPopSeg* pf);
/*!
 * \brief  destructeur
 *
 */
	~PushPopCover();

/*!
 * \brief  pointeur vers le premier point du contour
 *
 */
	SegIterator begin() const;
/*!
 * \brief  pointeur vers le dernier point du contour
 *
 */
	SegIterator end() const;

};

      
#endif
