#ifndef PushPopSegIterator_H
#define PushPopSegIterator_H

#include "AbstractSegIterator.h"
#include "ContourIterator.h"
#include "FactoryPushPopSeg.h"

/*!
 * \file PushPopSegIterator.h
 * \brief  definition d'un iterateur de segments maximaux avec ajout a l'avant et retrait a l'arriere
 * \author Tristan Roussillon
 *
 */

/*!
 * \class PushPopSegIterator
 * \brief iterateur de segments maximaux sur un contour avec ajout a l'avant et retrait a l'arriere
 * \author Tristan Roussillon
 *
 */
class PushPopSegIterator: public AbstractSegIterator {

protected : 

FactoryPushPopSeg* fseg; /*!< fabrique du seg */
std::vector<PushPopSeg*>* v; /*!< pointeur vers un ensemble de segments */

public : 

/*!
 * \brief Constructeur
 * \param pf : fabrique du segment
 * \param pv : ensemble de segments
 * \param pi : iterateur de contour sur le premier point du segment
 * \param pj : iterateur de contour sur le dernier point du segment
 */
 PushPopSegIterator(FactoryPushPopSeg* pf, 
											std::vector<PushPopSeg*>* pv, 
											const ContourIterator& pi, 
											const ContourIterator& pj);

/*!
 * \brief Destructeur
 */
	virtual ~PushPopSegIterator();
/*!
 * \brief Methode permettant
 * d'acceder au segment
 */
	Seg* operator*();
/*!
 * \brief Methode permettant
 * d'acceder au segment
 */
	Seg* getSeg();
/*!
 * \brief Methode permettant
 * d'avancer l'iterateur
 */
	virtual void operator++() = 0;

};

      
#endif
