#ifndef Seg_H
#define Seg_H

#include "Point.h"
#include "UIfig.h"
#include <string>
#include <sstream>
/*!
 * \file Seg.h
 * \brief Definition d'une classe modelisant un segment (portion) d'une courbe discrete
 * \author Tristan Roussillon
 *
 */


/*!
 * \class Seg
 * \brief segment (portion) d'une courbe discrete
 * Cette classe, abstraite, modelise un segment (portion) d'une courbe discrete,
 * pouvant etre aussi bien un segment de droite, qu'un arc de cercle, etc.
 * \author Tristan Roussillon
 */
class Seg {

protected : 

	//premier et dernier point du segment
	Point<int> first;/*!< premier point du segment */
	Point<int> last; /*!< dernier point du segment */


public :
/*!
 * \brief Methode virtuelle pure, 
 * permettant d'ajouter un point 
 * a l'avant du segment 
 * (dans le sens de parcours de la courbe)
 * 
 * \param p : point
 *
 */
	virtual bool push_front(const Point<int>& ) = 0;
/*!
 * \brief Methode 
 * retournant le dernier point 
 * du segment 
 */
	Point<int> getLast() const;
/*!
 * \brief Methode 
 * retournant le premier point 
 * du segment
 */
	Point<int> getFirst() const;

/*!
 * \brief Operateur egalite
 */
	bool operator==(const Seg& s) const; 

/*!
 * \brief Operateur difference
 */
	bool operator!=(const Seg& s) const; 


/*!
 * \brief Methode virtuelle pure, 
 * retournant la representation 
 * chaine de caractere du segment
 *
 */
	virtual std::string toString() const = 0;

	virtual ~Seg() {}
};

#endif
