#ifndef SegIterator_H
#define SegIterator_H

#include "ContourIterator.h"
#include "AbstractSegIterator.h"
#include "Seg.h"

/*!
 * \file SegIterator.h
 * \brief  definition d'un iterateur glouton de segments
 * \author Tristan Roussillon
 *
 */

/*!
 * \class SegIterator
 * \brief iterateur glouton de segments
 * \author Tristan Roussillon
 *
 */
class SegIterator {

private : 

	AbstractSegIterator* i; /*!< pointeur vers un iterateur */

public : 

/*!
 * \brief Constructeur 
 * \param pi : pointeur vers un iterateur de segments
 */
	SegIterator(AbstractSegIterator* pi); 

/*!
 * \brief Destructeur 
 */
	~SegIterator();

/*!
 * \brief Methode permettant
 * d'acceder au segment
 */
	Seg* operator*();

/*!
 * \brief Methode permettant
 * d'acceder a un iterateur
 * pointant sur le point
 * du contour suivant le dernier
 * point du segment
 */
	ContourIterator getFront();

/*!
 * \brief Methode permettant
 * d'acceder a un iterateur
 * pointant sur le premier 
 * point du segment
 */
	ContourIterator getBack();

/*!
 * \brief Methode permettant
 * d'avancer l'iterateur
 */
	void operator++();

/*!
 * \brief operateur d'egalite
 */ 
	bool operator==(const SegIterator& t) const;
/*!
 * \brief operateur de difference
 */ 
	bool operator!=(const SegIterator& t) const; 
};

      
#endif
