#ifndef WPOINT_H
   #define WPOINT_H

#include "Point.h"

/*!
 * \file WPoint.h
 * \brief  dfinition et implementation de la classe WPoint (un point 2D avec une ponderation)
 * \author Tristan Roussillon
 *
 */

/*!
 * \class WPoint
 * \brief  Classe generique pour traiter des points 2D avec une ponderation
 * \author Tristan Roussillon
 *
 */
template <class T> 
class WPoint : public Point<T> {

public:
	int k;/*!< ponderation*/
/*!
 * \brief constructeur par defaut
 */
	WPoint();
/*!
 * \brief constructeur
 */ 
	WPoint(T c1, T c2, int k);
/*!
 * \brief constructeur
 */ 
	WPoint(const Point<T>& p, int k);
/*!
 * \brief constructeur de copie
 */ 
	WPoint(const WPoint<T>&); /*!< constructeur de copie*/
/*!
 * \brief operateur d'affectation
 */ 
	WPoint<T>& operator=(const WPoint<T>&);
/*!
 * \brief operateur d'ecriture
 */ 
	std::ostream& operator<<(const WPoint<T>& p);
/*!
 * \brief accesseur du point
 */ 
	Point<T> getPoint();
};

//implementation du constructeur par defaut
template <class T> WPoint<T>::WPoint() {}

//implementation du constructeur
template <class T> WPoint<T>::WPoint(T c1, T c2, int z) {
          this->x = c1;
          this->y = c2;
					this->k = z;
}
template <class T> WPoint<T>::WPoint(const Point<T>& p, int z) {
          this->x = p.x;
          this->y = p.y;
					this->k = z;
}

//constructeur de copie
//template <class T> WPoint<T>::WPoint(const WPoint<T>& p) : x(p.x), y(p.y), k(p.k) {}
template <class T> WPoint<T>::WPoint(const WPoint<T>& p) {
          this->x = p.x;
          this->y = p.y;
					this->k = p.k;
}

//operateur d'affectation
template <class T> WPoint<T>& WPoint<T>::operator=(const WPoint<T>& p) {
          this->x = p.x;
          this->y = p.y;
					this->k = p.k;
          return *this;
}
//surcharge de l'operateur d eriture
template <class T> std::ostream& operator<<(std::ostream& os, const WPoint<T>& p) {
	os << "(" << p.x << "," << p.y << ")" << p.k << " ";
	return os;
}

template <class T> 	Point<T> WPoint<T>::getPoint() {
	return Point<T>(this->x,this->y);
}

   #endif


