#include "../inc/UIfig.h"

#include "../inc/Contour.h"
#include "../inc/ContourIterator.h"

#include "../inc/BackFrontCover.h"
#include "../inc/SegIterator.h"

#include "../inc/Seg.h"
#include "../inc/FactoryDSS4.h"


#include <string>
#include <sstream>
#include <exception>

/*!
 * \file CCP.cpp
 * \brief Programme qui lit a l'entree standard le code d'un contour,
 * qui decompose ce contour en parties convexes et concaves,
 * puis qui cree le graphique resultat "ccp.svg"
 *
 * \author Tristan Roussillon
 *
 */

int main(int argc, char *argv[]) {

	std::string curve, codeFreeman, ptx, pty;

	//lecture de la courbe
	std::cin >> curve;
	Contour *c = new Contour(curve);

	//creation de la figure
	UIfig *fig;
	fig = new UIfig("ccp.svg", c->getPts());

	//decomposition en segments maximaux
	FactoryDSS4 *factory = new FactoryDSS4();
	BackFrontCover *decomp = new BackFrontCover(c, factory);
	for (SegIterator i = decomp->begin(); i != decomp->end(); ++i) {
		DSS4* seg = static_cast<DSS4*> (*i);

		//iterateurs avant et arriere
		ContourIterator front = i.getFront();
		ContourIterator back = i.getBack();
		--back;

		//classification
		std::string color;
		try {
			if ( (seg->r(*back)<=-1)&&(seg->r(*front)<=-1) ) { //concave
				color = "green";
			} else if ( (seg->r(*back)>=1)&&(seg->r(*front)>=1) ) { //convex
				color = "blue";
			} else if ( (seg->r(*back)>=1)&&(seg->r(*front)<=-1) ) { //convex to concave
				color = "yellow";
			} else if ( (seg->r(*back)<=-1)&&(seg->r(*front)>=1) ) { //concave to convex
				color = "yellow";
			} else {
				color = "red";
			}
		} catch (std::out_of_range r) { //extremites des courbes ouvertes
			color = "black";
		} catch (std::exception e) {
			color = "red";
		}

		//dessin
		fig->addSegFromLeaningPoints(
			seg->getFirst(),
			seg->getLast(),
			seg->getUf(),
			seg->getUl(),
			seg->getLf(),
			seg->getLl(),
			color );	
				
	}



	//fin
	delete(decomp);
	delete(factory);
	delete(fig);
	delete(c);

}
