#include "../inc/UIfig.h"

#include "../inc/Contour.h"

#include "../inc/PushPopCover.h"
#include "../inc/SegIterator.h"

#include "../inc/Seg.h"
#include "../inc/FactoryDSS4.h"


#include <string>
#include <sstream>

/*!
 * \file Cover.cpp
 * \brief Programme qui lit a l'entree standard le code d'un contour,
 * qui decompose ce contour en segments maximaux,
 * puis qui cree le graphique resultat "cov.svg"
 *
 * \author Tristan Roussillon
 *
 */

#define FIG_SEGARC

#define TIME

#ifdef TIME
	#include <stdio.h>
	#include <sys/resource.h>

//fonctions
int cputime () {
  struct rusage rus;
  getrusage (0, &rus);
  return rus.ru_utime.tv_sec * 1000 + rus.ru_utime.tv_usec / 1000;
}

#endif

int main(int argc, char *argv[]) {

	std::string curve, codeFreeman, ptx, pty;

	//lecture de la courbe
	std::cin >> curve;
	Contour *c = new Contour(curve);

	//creation de la figure
	#ifdef FIG_SEGARC
	UIfig *fig;
	fig = new UIfig("cov.svg", c->getPts());
	#endif

	//depart du chrono
	#ifdef TIME
	int st;
	st = cputime(); 
	#endif

	//decomposition en segments maximaux
	FactoryDSS4 *factory = new FactoryDSS4();
	PushPopCover *decomp = new PushPopCover(c, factory);
	for (SegIterator i = decomp->begin(); i != decomp->end(); ++i) {
		DSS4* seg = static_cast<DSS4*> (*i);
		#ifdef FIG_SEGARC
		std::cerr << seg->toString() << std::endl;
		fig->addSegFromLeaningPoints(
			seg->getFirst(),
			seg->getLast(),
			seg->getUf(),
			seg->getUl(),
			seg->getLf(),
			seg->getLl(),
			"red" );	
		#endif				
	}

	//arret du chrono
	#ifdef TIME
	st = cputime() - st;
	std::cout << c->getLength() << ";";
	std::cout << st << std::endl;
	#endif

	//fin
	delete(decomp);
	delete(factory);
	#ifdef FIG_SEGARC
	delete(fig);
	#endif
	delete(c);

}
