#include "../inc/BruteForceCover.h"
#include "../inc/BruteForceSegIterator4ClosedContour.h"
#include "../inc/BruteForceSegIterator4OpenContour.h"

BruteForceCover::BruteForceCover(Contour* pc, FactorySeg* pf) {

	if (pc->isOpen()) { //si le contour est incomplet

		//iterateur de debut et fin du contour
		ContourScan* scan = new ContourScan(pc);
		ContourIterator i = scan->begin();
		ContourIterator end = scan->end();
		delete(scan);

		//iterateur de debut et fin du segment
		ContourIterator j = i;
		if (i != end) ++j;
		else throw BadConnectivity();	//un contour a forcement au moins deux points

		//calcul du premier segment
		vSegs.push_back( pf->createSeg(*i, *j) );	++j;
		while ( (j != end) && (vSegs.back()->push_front(*j)) ) ++j;
		this->firstSeg = new BruteForceSegIterator4OpenContour(pf,&vSegs,i,j,end);

		//init fin iterateur de segment 
		vTmp.push_back(0);
		this->lastSeg = new BruteForceSegIterator4OpenContour(pf,&vTmp,i,j,end); 

	} else { // si le contour est complet

		//iterateur de debut du contour
		ContourScan *scan = new ContourScan(pc);
		ContourIterator i = scan->begin();
		delete(scan);

		//iterateur de debut et fin du segment
		ContourIterator j = i;
		++j;

		//initialisation : premier seg max a l'avant
		vSegs.push_back( pf->createSeg(*i, *j) );	++j;
		while (vSegs.back()->push_front(*j)) ++j;

		if (*i != *j) {
			//si le premier segment ne recouvre pas tout le contour

			//calcul des segments partant des points suivants jusqu'a 
			//trouver le premier seg max a l'avant ET a l'arriere
			do {
				++i; j = i; ++j;
				vSegs.push_back( pf->createSeg(*i, *j) ); ++j;
				while (vSegs.back()->push_front(*j)) ++j;
			} while (vSegs.back()->getLast() == vSegs.at(0)->getLast());

		}
		this->firstSeg = new BruteForceSegIterator4ClosedContour(pf,&vSegs,i,j);

		//init fin iterateur de segment 
		vTmp.push_back(vSegs.back());
		this->lastSeg = new BruteForceSegIterator4ClosedContour(pf,&vTmp,i,j); 

	}
}

BruteForceCover::~BruteForceCover() {
	delete(this->firstSeg);
	delete(this->lastSeg);
	for (int i = 0; i < vSegs.size(); i++) {
		delete(vSegs.at(i));
	}
}

SegIterator BruteForceCover::begin() const {
	return SegIterator(this->firstSeg);
}

SegIterator BruteForceCover::end() const {
	return SegIterator(this->lastSeg);
}
