#include "../inc/DSS.h"

/******************************/
//implmentation de la classe

//constructeur 
DSS::DSS(const Point<int>& P1, const Point<int>& P2) {

	if (std::max(std::abs(P1.x-P2.x), std::abs(P1.y-P2.y))>1) {
 		throw BadConnectivity();
	}

	this->a = P2.y - P1.y;
	this->b = P2.x - P1.x;
	this->c1 = this->c2 = (a*P1.x - b*P1.y);
	this->Ul = this->Ll = P2;
	this->Uf = this->Lf = P1;
	this->first = P1;
	this->last = P2;
}

//constructeur de copie
DSS::DSS(const DSS& d) : a(d.a), b(d.b), c1(d.c1), c2(d.c2), Uf(d.Uf), Lf(d.Lf), Ul(d.Ul), Ll(d.Ll) {
	this->first = d.first;
	this->last = d.last;
}

//oprateur d'affectation
DSS& DSS::operator=(const DSS& d) {
	this->a = d.a;
	this->b = d.b;
	this->c1 = d.c1;
	this->c2 = d.c2;
	this->first = d.first;
	this->last = d.last;
	this->Uf = d.Uf;
	this->Lf = d.Lf;
	this->Ul = d.Ul;
	this->Ll = d.Ll;

	return *this;
}

/******************************/
//ajout d'un point M a un DSS
bool DSS::push(const Point<int>& M, Point<int>& rend, Point<int>& end, 
			Point<int>& pUf, Point<int>& pUl, Point<int>& pLf, Point<int>& pLl, 
			int pa, int pb, int pc1, int pc2) {

	int r = pa*M.x - pb*M.y;
	
	if (this->isBadConnected(rend, end, M)) return false;
	if ( (r < pc1-1)||(r > pc2+1) )
		return false; //M fortement ext
	else { 
		end = M; //on ajoute
		//mise a jour des pts d'appui (sur les references)
		if (r == pc1) pLl = M; //M faiblement interieur 
		if (r == pc2) pUl = M; //M faiblement interieur 
		if (r == pc1-1) {	//M faiblement exterieur
			pLl = M; //dernier point d'appui inf (= M)
			pUf = pUl; //premier point d'appui sup
		}
		if (r == pc2+1) { //M faiblement exterieur
			pUl = M; //dernier point d'appui sup (= M)
			pLf = pLl; //premier point d'appui inf
		}
		//mise a jour des parametres (sur les valeurs)
		if (this->Uf==this->Ul) { 
			this->a = (this->Ll.y - this->Lf.y);
			this->b = (this->Ll.x - this->Lf.x);
			this->c1 = this->a*this->Ll.x - this->b*this->Ll.y; 
			this->c2 = this->a*this->Ul.x - this->b*this->Ul.y; 
		}
		if (this->Lf==this->Ll) {
			this->a = (this->Ul.y - this->Uf.y);
			this->b = (this->Ul.x - this->Uf.x);
			this->c1 = this->a*this->Ll.x - this->b*this->Ll.y; 
			this->c2 = this->a*this->Ul.x - this->b*this->Ul.y; 	
		}
		return true;
	}
}

//ajout d'un point M devant un DSS
bool DSS::push_front(const Point<int>& M) {
	return this->push(M, this->first, this->last, 
				this->Uf, this->Ul, this->Lf, this->Ll, 
				this->a, this->b, this->c1, this->c2);
}

//ajout d'un point M derriere un DSS
bool DSS::push_back(const Point<int>& M) {
	return this->push(M, this->last, this->first, 
				this->Ll, this->Lf, this->Ul, this->Uf, 
				-this->a, -this->b, -this->c2, -this->c1);
}

//mise a jour des caracteristiques
//lors de la suppression d'un point
void DSS::update(const Point<int>& P0, const Point<int>& Pn, 
			Point<int>& U1, Point<int>& U2, Point<int>& L1, Point<int>& L2,
			const Vecteur<int>& v, int k) {

	if ((v.y != 0)&&(v.x != 0)) { 
		//nb de fois que v est translate jusqu'au bout du segment
		int kU = std::min((U2.x-P0.x)/v.x,(U2.y-P0.y)/v.y); 
		int kL = std::min(((Pn.x-L1.x)/v.x),((Pn.y-L1.y)/v.y)); 
		U1 = U2 + (-v*kU);
		L2 = L1 + v*kL;
	} else {
		if (v.y == 0) {
			L2 = L1 + v*((Pn.x-L1.x)/v.x);
			U1 = L1;
			U2 = L2;	
		}
		if (v.x == 0) {
			L2 = L1 + v*((Pn.y-L1.y)/v.y);
			U1 = L1;
			U2 = L2;	
		}
	}	

	//a, b sont les coordonnees du vecteur v
	this->a = k*v.y;
	this->b = k*v.x;
	//
	this->c1 = this->a*this->Ll.x - this->b*this->Ll.y; 
	this->c2 = this->a*this->Ul.x - this->b*this->Ul.y; 
}
/******************************/
//supprimer un point M d'un DSS
bool DSS::pop(const Point<int>& M, Point<int>& pfirst, Point<int>& plast, 
			Point<int>& pUf, Point<int>& pUl, Point<int>& pLf, Point<int>& pLl,
			int k) {

	if (M != pfirst) {
 		throw BadConnectivity();
	}

	//s'il ne reste que 2 pts, on ne supprime pas
	if (this->areNeighbors(pfirst, plast)) {
		return false;
	} else {
		//mettre a jour first
		pfirst = nextPt(pfirst, plast);

		//si le premier point est un point d'appui sup...
		if (M == pUf) {
			//...et que les premier et dernier points d'appui inf sont confondus
			if (pLf == pLl) {
				Vecteur<int> vect = (pLf - (pUf + this->shift()));
				this->update(pfirst, plast, pUf, pUl, pLf, pLl, vect, k);
			} else {
				Vecteur<int> v(this->b,this->a);
				pUf = pUf + v*k;
			}
		}

		//si le premier point est un point d'appui inf...
		if (M == pLf) {
			//...et que les premier et dernier points d'appui sup sont confondus
			if (pUf == pUl) { 
				Vecteur<int> vect = (pUf - (pLf - this->shift()));			
				this->update(pfirst, plast, pLf, pLl, pUf, pUl, vect, k);
			} else {
				Vecteur<int> v(this->b,this->a);
				pLf = pLf + v*k;
			}
		}
		return true;
	}
}
/******************************/
//supprimer un point M derriere un DSS
bool DSS::pop_back(const Point<int>& M) {
	return this->pop(M, this->first, this->last, 
			this->Uf, this->Ul, this->Lf, this->Ll, 1);
}
//supprimer un point M devant un DSS
bool DSS::pop_front(const Point<int>& M) {
	return this->pop(M, this->last, this->first, 
			this->Ul, this->Uf, this->Ll, this->Lf, -1);
}

bool DSS::isInside(const Point<int>& M) {

	if (this->isBadConnected(this->first, this->last, M)) return false;
	else {

		int r = this->a*M.x - this->b*M.y;
		if ( (r < this->c1-1)||(r > this->c2+1) ) return false;
		else return true;

	}

}

/******************************/
std::string DSS::toString() const {
	std::ostringstream output;
	
	output << "("<< this->a << "/" << this->b << ", " << this->c1 << ", " << this->c2 << ") ";
	output << "first " << this->first << "last " << this->last;
	output << "Uf " << this->Uf << "Ul " << this->Ul << "Lf " << this->Lf << "Ll " << this->Ll;

	return output.str();	
}

//surcharge de l'oprateur de sortie
std::ostream& operator<<(std::ostream& ostr, const DSS& d) {
	return ostr << d.toString();
}


/******************************/
//accesseurs
int DSS::getA() {
	return this->a;
}
int DSS::getB() {
	return this->b;
}
int DSS::getC1() {
	return this->c1;
}
int DSS::getC2() {
	return this->c2;
}
int DSS::getC(bool ccw) {
	if (ccw) return this->c1;
	else return this->c2;
}
int DSS::getMu() {
	return this->c1;
}
double DSS::getSlope() {
	return this->a/this->b;
}
double DSS::getIntercept() {
	return this->c1/this->b;
}

int DSS::r(const Point<int>& pt) {
	return (this->getA()*pt.x-this->getB()*pt.y-this->getC1());	
}

Point<int> DSS::getUf() const {
	return this->Uf;
}
Point<int> DSS::getUl() const{
	return this->Ul;
}
Point<int> DSS::getLf() const{
	return this->Lf;
}
Point<int> DSS::getLl() const{
	return this->Ll;
}

