#include "../inc/DSS8.h"

//constructeur 
DSS8::DSS8(const Point<int>& P1, const Point<int>& P2) : DSS(P1,P2) {}


int DSS8::getOmega() const {
	return std::max(std::abs(this->a), std::abs(this->b));
}

bool DSS8::areNeighbors(const Point<int>& P, const Point<int>& Q) const {
	return (std::max(std::abs(P.x-Q.x), std::abs(P.y-Q.y))<=1);
}

bool DSS8::isBadConnected(const Point<int>& pfirst, const Point<int>& plast, const Point<int>& N) const {
	if (!areNeighbors(plast,N)) {
 		throw BadConnectivity();
	}
	//si presence de 4-connexite, le seg ne peut pas etre exactement 8-connexe
	if (std::abs(this->b) > std::abs(this->a)) {
		if ((plast.x==N.x)&&(std::abs(plast.y-N.y)==1)) return true;
	}
	if (std::abs(this->b) < std::abs(this->a)) {
		if ((plast.y==N.y)&&(std::abs(plast.x-N.x)==1)) return true;
	}
	if (this->b == this->a) {
		if ((plast.x - pfirst.x) > 0) {
			if ((plast.x==N.x) && ((N.y-plast.y) < 0)) return true; 
			if ((plast.y==N.y) && ((N.x-plast.x) < 0)) return true;
		} else {
			if ((plast.x==N.x) && ((N.y-plast.y) > 0)) return true; 
			if ((plast.y==N.y) && ((N.x-plast.x) > 0)) return true;//ok
		} 
	}
	if (this->b == -this->a) {
		if ((plast.x - pfirst.x) > 0) {
			if ((plast.x==N.x) && ((N.y-plast.y) > 0)) return true; 
			if ((plast.y==N.y) && ((N.x-plast.x) < 0)) return true;
		} else {
			if ((plast.x==N.x) && ((N.y-plast.y) < 0)) return true; 
			if ((plast.y==N.y) && ((N.x-plast.x) > 0)) return true;//ok
		} 
	}
	return false;
}

/******************************/
Point<int> DSS8::nextPt(const Point<int>& pfirst, const Point<int>& plast) const {
	Point<int> Q = pfirst;
	int dx = (plast.x - pfirst.x);
	int dy = (plast.y - pfirst.y);
	if (std::abs(this->b) >= std::abs(this->a)) {
		if (dx > 0) Q.x++;
		if (dx < 0) Q.x--;
		int r = this->a*Q.x-this->b*Q.y;
		if (((r-c1)>=0)&&((r-c2)<=0)) return Q;
		else {
			if (dy > 0) Q.y++;
			if (dy < 0) Q.y--;
			return Q;
		}
	} else {
		if (dy > 0) Q.y++;
		if (dy < 0) Q.y--;
		int r = this->a*Q.x-this->b*Q.y;
		if (((r-c1)>=0)&&((r-c2)<=0)) return Q;
		else {
			if (dx > 0) Q.x++;
			if (dx < 0) Q.x--;
			return Q;
		}
	}
}

/******************************/
//vecteur dplcmt entre le pt de Bezout du seg
//et son le pt d'appui correspondant
Vecteur<int> DSS8::shift() const { 
	Vecteur<int> v;
	if (std::abs(this->b) >= std::abs(this->a)) {
		if (this->b >= 0) v = Vecteur<int>(0, 1); //a <=> b ok comme ca
		else v = Vecteur<int>(0, -1);
	} else {
		if (this->a >= 0) v = Vecteur<int>(-1, 0);
		else v = Vecteur<int>(1, 0);				
	}
	return v;
}

