#include "../inc/FP.h"
#include <exception>

bool quadrant(Vecteur<int> v, int q) {
	if (q == 1) {
		return ( (v.x>=0)&&(v.y>=0) );
	} else if (q == 2) {
		return ( (v.x>=0)&&(v.y<=0) );
	} else if (q == 3) {
		return ( (v.x<=0)&&(v.y<=0) );
	} else if (q == 4) {
		return ( (v.x<=0)&&(v.y>=0) );
	} else {
		std::cerr << "wrong quadrant number" << std::endl;
		throw std::exception();
		return false;
	}
}

Vecteur<double> getShift(Point<int> a, Point<int> b, Point<int> c) {

	Vecteur<double> shift;

	Vecteur<int> e1 = b - a;
	Vecteur<int> e2 = c - b;

	if ( (e1.x*e2.y-e1.y*e2.x) <= 0 ) {

		//convex turn
		if ( (quadrant(e1,1))&&(quadrant(e2,1)) ) {
			shift = Vecteur<double>(0.5,-0.5);
		} else if ( (quadrant(e1,2))&&(quadrant(e2,2)) ) {
			shift = Vecteur<double>(-0.5,-0.5);
		} else if ( (quadrant(e1,3))&&(quadrant(e2,3)) ) {
			shift = Vecteur<double>(-0.5,0.5);
		} else if ( (quadrant(e1,4))&&(quadrant(e2,4)) ) {
			shift = Vecteur<double>(0.5,0.5);
		} else {
			std::cerr << "Not valid polygon" << std::endl;
			shift = Vecteur<double>(0,0);
		}

	} else {

		//concave turn
		if ( (quadrant(e1,1))&&(quadrant(e2,1)) ) {
			shift = Vecteur<double>(-0.5,0.5);
		} else if ( (quadrant(e1,2))&&(quadrant(e2,2)) ) {
			shift = Vecteur<double>(0.5,0.5);
		} else if ( (quadrant(e1,3))&&(quadrant(e2,3)) ) {
			shift = Vecteur<double>(0.5,-0.5);
		} else if ( (quadrant(e1,4))&&(quadrant(e2,4)) ) {
			shift = Vecteur<double>(-0.5,-0.5);
		} else {
			std::cerr << "Not valid polygon" << std::endl;
			shift = Vecteur<double>(0,0);
		}

	} 

	return shift;
}

std::vector<Point<int> > draw(int b, int a, int r,
															int dx, int dy,
															Point<int> P, Point<int> Q) {

	std::vector<Point<int> > res;
	while (P != Q) {
		if (r < b) {
			r = r + a;
			P.x = P.x + dx;
		} else {
			r = r - b;
			P.y = P.y + dy;
		}
		res.push_back(P);
	}

	return res;
}

std::vector<Point<int> > getEdge(Point<int> a, Point<int> b, Point<int> c, Point<int> d) {

	std::vector<Point<int> > res;

	Vecteur<int> e1 = b - a;
	Vecteur<int> e2 = c - b;
	Vecteur<int> e3 = d - c;

	if ( (e1.x*e2.y-e1.y*e2.x) <= 0 ) {
		if ( (e2.x*e3.y-e2.y*e3.x) <= 0 ) {
			//convex part
			res = getUpperEdge(b,c);
		} else {
			//from convex to concave			
			Point<int> b2;
			if ( (e2.x>0)&&(e2.y>=0) ) {
				b2 = b+Vecteur<int>(1,-1);
			} else if ( (e2.x>=0)&&(e2.y<0) ) {
				b2 = b+Vecteur<int>(-1,-1);
			} else if ( (e2.x<0)&&(e2.y<=0) ) {
				b2 = b+Vecteur<int>(-1,1);
			} else if ( (e2.x<=0)&&(e2.y>0) ) {
				b2 = b+Vecteur<int>(1,1);
			} 
			res = getLowerEdge(b2,c);
		}
	} else {
		if ( (e2.x*e3.y-e2.y*e3.x) <= 0 ) {
			//from concave to convex
			Point<int> b2;
			if ( (e2.x>=0)&&(e2.y>0) ) {
				b2 = b+Vecteur<int>(-1,1);
			} else if ( (e2.x>0)&&(e2.y<=0) ) {
				b2 = b+Vecteur<int>(1,1);
			} else if ( (e2.x<=0)&&(e2.y<0) ) {
				b2 = b+Vecteur<int>(1,-1);
			} else if ( (e2.x<0)&&(e2.y>=0) ) {
				b2 = b+Vecteur<int>(-1,-1);
			} 
			res = getUpperEdge(b2,c);
		} else {
			//concave part
			res = getLowerEdge(b,c);
		}
	} 

	return res;
}

std::vector<Point<int> > getUpperEdge(Point<int> p, Point<int> q) {

	std::vector<Point<int> > res;

	Vecteur<int> e = q - p;

	if (quadrant(e,1)) {
		res = draw(std::abs(e.x),std::abs(e.y),0,1,1,p,q);
	} else if (quadrant(e,2)) {
		res = draw(std::abs(e.x),std::abs(e.y),
							(std::abs(e.x)+std::abs(e.y)-1),1,-1,p,q);
	} else if (quadrant(e,3)) {
		res = draw(std::abs(e.x),std::abs(e.y),0,-1,-1,p,q);
	} else if (quadrant(e,4)) {
		res = draw(std::abs(e.x),std::abs(e.y),
							(std::abs(e.x)+std::abs(e.y)-1),-1,1,p,q);
	}

	return res;
}

std::vector<Point<int> > getLowerEdge(Point<int> p, Point<int> q) {

	std::vector<Point<int> > res;

	Vecteur<int> e = q - p;

	if (quadrant(e,1)) {
		res = draw(std::abs(e.x),std::abs(e.y),
							(std::abs(e.x)+std::abs(e.y)-1),1,1,p,q);
	} else if (quadrant(e,2)) {
		res = draw(std::abs(e.x),std::abs(e.y),
							0,1,-1,p,q);
	} else if (quadrant(e,3)) {
		res = draw(std::abs(e.x),std::abs(e.y),
							(std::abs(e.x)+std::abs(e.y)-1),-1,-1,p,q);
	} else if (quadrant(e,4)) {
		res = draw(std::abs(e.x),std::abs(e.y),
							0,-1,1,p,q);
	}

	return res;
}

/*!
 * \file FP.cpp
 * \brief Implementation de la classe FP calculant
 * le polygone reversible et respectant les 
 * parties convexes et concaves du contour
 * en utilisant les methodes
 * push_front et pop_back de la classe DSS4
 * \author Tristan Roussillon
 *
 */

//constructeur 
FP::FP(Contour* pc) {

///////////////////////
//// INITIALISATION ///
///////////////////////

	//std::cout << "init" << std::endl;

	//scan initialisation
	ContourScan *scan = new ContourScan(pc);
	ContourIterator i = scan->begin();
	ContourIterator end = scan->end();
	delete(scan);
	ContourIterator k = i; //beginning of the current DSS
	ContourIterator l = k; //end of the current DSS
	if (l == end) throw BadConnectivity();
	else ++l;

	//dss initialisation
	DSS* seg = new DSS4(*k,*l);
	AdapterDSS *aseg; //dss adapter

	Point<int> tmpUf = seg->getUf();
	Point<int> tmpLf = seg->getLf();

	//initialisation of the list of vertices 
	std::list<Point<int> > vTmpU, vTmpL;
	vTmpU.push_back(tmpUf);
	vTmpL.push_back(tmpLf);

	if (l == end) throw BadConnectivity();
	else ++l;
	if (!pc->isOpen()) ++end;
	//first adding step
	while ( (l != end)&&(seg->push_front(*l)) ) {
		//store the first upper leaning point
		if (seg->getUf() != tmpUf) {
			tmpUf = seg->getUf();
			vTmpU.push_back(tmpUf);
		}
		//store the first lower leaning point
		if (seg->getLf() != tmpLf) {
			tmpLf = seg->getLf();
			vTmpL.push_back(tmpLf);
		}
		++l; //move forward
	}

	//convexity initialisation
	bool isConvex;
	if (l != end) {

		if (seg->r(*l)<-1) { //convex part
			this->vPts = vTmpU;
			isConvex = true;
			aseg = new AdapterDSS1(seg);
		} else {                     //concave part
			this->vPts = vTmpL;
			isConvex = false;
			aseg = new AdapterDSS0(seg);
		}

	} else {
		//the part is assumed to be convex
		//if it is straight
		this->vPts = vTmpU;
		isConvex = true;
		aseg = new AdapterDSS1(seg);
	}


////////////////
///// BODY /////
////////////////

	//std::cout << "body" << std::endl;

	while (l != end) {

		//store the last leaning point 
		//if the first and last leaning points 
		//of the MS are not confounded
		if (aseg->getXf() != aseg->getXl()) {
			this->vPts.push_back(aseg->getXl());
		}

		//removing step
		while (!aseg->isInside(*l)) {
			//remove a point from the back
			aseg->pop_back(*k); ++k; 
			//store the last leaning point
			if (aseg->getXl() != this->vPts.back()) {
				this->vPts.push_back(aseg->getXl());
			}
		}

		//remove the last leaning point 
		//if the first and last leaning points 
		//of the dss are not confounded
		if (aseg->getXf() != aseg->getXl()) {
			this->vPts.pop_back();
		}		

		//adding step
		while ( (l != end)&&(aseg->push_front(*l)) ) {
			//store the first leaning point
			if (aseg->getXf() != this->vPts.back()) {
				this->vPts.push_back(aseg->getXf());
			}
			++l; //move forward
		}

		//transition step
		if (l != end) {
			if ( (isConvex)&&(aseg->r(*l)>1) ) {
				//from convex to concave
				isConvex = false;
				aseg = new AdapterDSS0(seg);
			} else if ( (!isConvex)&&(aseg->r(*l)<-1) ) {
				//from concave to convex
				isConvex = true;
				aseg = new AdapterDSS1(seg);
			}
		}

	}

////////////////
///// END //////
////////////////

	//std::cout << "end" << std::endl;

	//last removing step
	while (aseg->pop_back(*k)) {
		++k; //remove a point from the back
		//store the last leaning point
		if (aseg->getXl() != this->vPts.back()) {
			this->vPts.push_back(aseg->getXl());
		}
	}

	//destroy seg
	delete seg;
}

FP::FP(const std::vector<Point<int> >& pv) {
	this->vPts.assign( pv.begin(), pv.end() );
} 

std::vector<Point<int> > FP::getFP() const {

	std::vector<Point<int> > res;
	res.assign( this->vPts.begin(),  this->vPts.end() );
	return res;

}

std::vector<Point<double> > FP::getMLP() const {

	std::vector<Point<double> > mlp;
	std::vector<Point<int> > polyg = this->getFP();
	int n = polyg.size()-1;
	bool isOpen;
	if (polyg.at(0) == polyg.at(n)) isOpen = false; 
	else isOpen = true;

	//begin
	if (isOpen) {
		//the first point is not translated
		mlp.push_back(polyg.at(0).castindouble());
	} else {	
		//the first point is translated
		Point<double> p = polyg.at(0).castindouble();
		Vecteur<double> s = getShift(polyg.at(n-1),polyg.at(0),polyg.at(1));
		mlp.push_back(p+s);
	}

	//for all pair of consecutives edges
	for (int i = 1; i < n; i++) {
		Point<double> p = polyg.at(i).castindouble();
		Vecteur<double> s = getShift(polyg.at(i-1),polyg.at(i),polyg.at(i+1));
		mlp.push_back(p+s);
	} 

	//end
	if (isOpen) {
		//the last point is not translated
		mlp.push_back(polyg.at(n).castindouble());
	} else {	
		//the first point is repeated
		mlp.push_back(mlp.at(0));
	}

	return mlp;
}

Contour FP::getContour() const {

	std::vector<Point<int> > contour; 
	std::vector<Point<int> > polyg = this->getFP();
	int n = polyg.size()-1;
	bool isOpen;
	if (polyg.at(0) == polyg.at(n)) isOpen = false; 
	else isOpen = true;

	//the first point
	contour.push_back(polyg.at(0));

	//the first edge
	if (isOpen) {
		Vecteur<int> e1 = polyg.at(1) - polyg.at(0);
		Vecteur<int> e2 = polyg.at(2) - polyg.at(1);
		if ( (e1.x*e2.y-e1.y*e2.x) <= 0 ) {
			//convex turn
			std::vector<Point<int> > edge = getUpperEdge(polyg.at(0),
																									 polyg.at(1));
			for (int k = 0; k < edge.size(); k++) {
				contour.push_back(edge.at(k));
			}			
		} else {
			//concave turn
			std::vector<Point<int> > edge = getLowerEdge(polyg.at(0),
																									 polyg.at(1));
			for (int k = 0; k < edge.size(); k++) {
				contour.push_back(edge.at(k));
			}			
		}
	} else {	
		std::vector<Point<int> > edge = getEdge(polyg.at(n-1),
																						polyg.at(0),
																						polyg.at(1),
																						polyg.at(2));
		for (int k = 0; k < edge.size(); k++) {
			contour.push_back(edge.at(k));
		}
	}


	//for all pair of consecutives edges
	for (int i = 1; i < (n-1); i++) {
		std::vector<Point<int> > edge = getEdge(polyg.at(i-1),
																						polyg.at(i),
																						polyg.at(i+1),
																						polyg.at(i+2));
		for (int k = 0; k < edge.size(); k++) {
			contour.push_back(edge.at(k));
		}
	} 

	//the last edge
	if (isOpen) {
		Vecteur<int> e1 = polyg.at(n-1) - polyg.at(n-2);
		Vecteur<int> e2 = polyg.at(n) - polyg.at(n-1);
		if ( (e1.x*e2.y-e1.y*e2.x) <= 0 ) {
			//convex turn
			std::vector<Point<int> > edge = getUpperEdge(polyg.at(n-1),
																									 polyg.at(n));
			for (int k = 0; k < edge.size(); k++) {
				contour.push_back(edge.at(k));
			}			
		} else {
			//concave turn
			std::vector<Point<int> > edge = getLowerEdge(polyg.at(n-1),
																									 polyg.at(n));
			for (int k = 0; k < edge.size(); k++) {
				contour.push_back(edge.at(k));
			}			
		}
	} else {	
		std::vector<Point<int> > edge = getEdge(polyg.at(n-2),
																						polyg.at(n-1),
																						polyg.at(n),
																						polyg.at(1));
		for (int k = 0; k < edge.size(); k++) {
			contour.push_back(edge.at(k));
		}
	}

	return Contour(contour);
}

