#include "../inc/Fraction.h"
#include <sstream>
#include <cmath>


//implementation des constructeurs par leur liste d'initialisation
Fraction::Fraction(int num, int den) : _num(num), _den(den) {
	//le denominateur est tjrs positif
	if (_den < 0) {
		_num = -_num;
		_den = -_den;
	}
}
Fraction::Fraction(const Fraction& f) : _num(f._num), _den(f._den) {}

//destructeur
Fraction::~Fraction() {}

//oprateur d'affectation
Fraction& Fraction::operator=(const Fraction& f) {
          _num = f._num;
          _den = f._den;
          return *this;
}

//implementation des accesseurs
int Fraction::num() const {
    return _num;
}
int Fraction::den() const {
    return _den;
}

//calcul du plus grand diviseur commun
int Fraction::pgcd(int a, int b) {
	
	//variables locales
	int grand, petit;
	
	//determination du plus grand et du plus petit entier
	if (std::abs(a) > std::abs(b)) {
		grand = std::abs(a);
		petit = std::abs(b);
	} else {
		grand = std::abs(a);
		petit = std::abs(b);
	}

	//cas particuliers
	if (petit == grand) return petit;
	else
	if (petit == 0) return grand;
	else
	if (petit == 1) return petit;
	else {
		//calcul du plus grand diviseur commun par l'algorithme d'Euclide
		int reste;
		do {
			reste = grand%petit;
			grand = petit;
			petit = reste;
		} while (reste >= 1);
		return grand;
	}
}

//pgcd
int Fraction::pgcd() {
	return pgcd(_num, _den);
}

//simplification
void Fraction::reduire() {
	int diviseur = pgcd(_num, _den);
	if (diviseur > 1) {
		_num = _num / diviseur;
		_den = _den / diviseur;
	}
}

//implementation des operateurs de comparaison
bool Fraction::operator<(const Fraction& f) const {
     return (f._den * _num < f._num * _den);
}
bool Fraction::operator>(const Fraction& f) const {
     return (f._den * _num > f._num * _den);
}
bool Fraction::operator<=(const Fraction& f) const {
     return (f._den * _num <= f._num * _den);
}
bool Fraction::operator>=(const Fraction& f) const {
     return (f._den * _num >= f._num * _den);
}
bool Fraction::operator==(const Fraction& f) const {
     return (f._den * _num == f._num * _den);
}
bool Fraction::operator!=(const Fraction& f) const {
     return (f._den * _num != f._num * _den);
}

//implementation des operateurs de calcul
Fraction Fraction::operator*(const Fraction& f) const {
	Fraction res(_num * f._num, _den * f._den);
	return res;
}
Fraction Fraction::inv() const {
	Fraction res(_den, _num);
	return res;
}
Fraction Fraction::abs() const {
	Fraction res(std::abs(_num),std::abs(_den));
	return res;
}
Fraction Fraction::operator/(const Fraction& f) const {
	Fraction res(_num * f._den, _den * f._num);
	return res;
}
Fraction Fraction::operator+(const Fraction& f) const {
	Fraction res;
	if (_den != f._den) {
		res._num = _num * f._den + f._num * _den;
		res._den = _den * f._den;
	} else {
		res._num = _num + f._num;
		res._den = _den;
	} 
	return res;
}
Fraction Fraction::operator-(const Fraction& f) const {
	Fraction res;
	if (_den != f._den) {
		res._num = _num * f._den - f._num * _den;
		res._den = _den * f._den;
	} else {
		res._num = _num - f._num;
		res._den = _den;
	} 
	return res;
}

//surcharge de l'operateur moins
Fraction Fraction::operator-() const {
	Fraction oppose(-this->_num, this->_den);
	return oppose;
}

//la fraction est-elle calculable ?
bool Fraction::estDef() const {
	return (_den!=0);
}

//calcul de la fraction
double Fraction::reel() const {
	return this->estDef()?((double) _num / _den):throw DivisionByZero();
}

//affichage
std::string Fraction::toString() const {
std::ostringstream output;
output << _num << "/" << _den;
return output.str();
}

//surcharge de l'oprateur de sortie
std::ostream& operator<<(std::ostream& ostr, const Fraction& f) {
	return ostr << f.toString();
}


