#include "../inc/PushPopCover.h"
#include "../inc/PushPopSegIterator4ClosedContour.h"
#include "../inc/PushPopSegIterator4OpenContour.h"

PushPopCover::PushPopCover(Contour* pc, FactoryPushPopSeg* pf) {

	if (pc->isOpen()) { //si le contour est incomplet

		//iterateur de debut et fin du contour
		ContourScan* scan = new ContourScan(pc);
		ContourIterator i = scan->begin();
		ContourIterator end = scan->end();
		delete(scan);

		//iterateur de debut et fin du segment
		ContourIterator j = i;
		if (i != end) ++j;
		else throw BadConnectivity();	//un contour a forcement au moins deux points

		//calcul du premier segment
		vSegs.push_back( pf->createSeg(*i, *j) );	++j;
		while ( (j != end) && (vSegs.back()->push_front(*j)) ) ++j;
		this->firstSeg = new PushPopSegIterator4OpenContour(pf,&vSegs,i,j,end);

		//init fin iterateur de segment 
		vTmp.push_back(0);
		this->lastSeg = new PushPopSegIterator4OpenContour(pf,&vTmp,i,j,end); 

	} else { // si le contour est complet

		//iterateur de debut du contour
		ContourScan *scan = new ContourScan(pc);
		ContourIterator i = scan->begin();
		delete(scan);

		//iterateur de debut et fin du segment
		ContourIterator j = i;
		++j;

		//initialisation : premier seg max a l'avant
		vSegs.push_back( pf->createSeg(*i, *j) );	++j;
		while (vSegs.back()->push_front(*j)) ++j;

		if (*i != *j) {
			//si le premier segment ne recouvre pas tout le contour

			//croissance vers l'arriere pour que le segment 
			//soit max a l'avant ET a l'arriere
			do {
				--i;
			} while (vSegs.back()->push_back(*i));
			++i;

		}
		this->firstSeg = new PushPopSegIterator4ClosedContour(pf,&vSegs,i,j);

		//init fin iterateur de segment 
		vTmp.push_back(vSegs.back());
		this->lastSeg = new PushPopSegIterator4ClosedContour(pf,&vTmp,i,j); 

	}
}

PushPopCover::~PushPopCover() {
	delete(this->firstSeg);
	delete(this->lastSeg);
	for (int i = 0; i < vSegs.size(); i++) {
		delete(vSegs.at(i));
	}
}

SegIterator PushPopCover::begin() const {
	return SegIterator(this->firstSeg);
}

SegIterator PushPopCover::end() const {
	return SegIterator(this->lastSeg);
}
