#include "../inc/UIfig.h"


//constructeur 
UIfig::UIfig(std::string s, const std::vector<Point<int> >& v) {

	//creation du fichier
	this->fic.open(s.c_str(), std::ios::out); //creer le fichier (qu'il existe ou non)
	if (!this->fic.is_open()) {
		std::cerr << "Impossible d'ouvrir le fichier " << s << " en ecriture" << std::endl;
		throw std::ios_base::failure("erreur d'ouverture en ecriture");
	}

	this->fic << "<svg version='1.1' baseProfile='full' xmlns='http://www.w3.org/2000/svg' id='svg' width='100%' height='100%' onload='handleLoad(evt)' onmousedown='Grab(evt)' onmouseup='Drop(evt)'> " << std::endl;

	//mise a l'echelle
	const int dx = 300;
	const int dy = 300;
	const int t = 10;

	this->fic << "<g id='all' transform='";
	this->fic << "translate(" << dx << "," << dy << ") ";
	this->fic << "scale(" << t << "," << -t << ")'>" << std::endl;

	//dessin du contour
	this->addPolyline(v, "contour", "black");
}

//destructeur
UIfig::~UIfig() {

	this->fic << "</g>" << std::endl;
/*
	std::ifstream ifs ("js.txt");
	std::string s;
	while (std::getline (ifs, s)) {
		this->fic << s << std::endl;
	}
*/
	//code javascript pour pouvoir
	//zoomer et translater la figure
	//quand le svg est vu d'un navigateur
	this->fic << "<script><![CDATA[ function handleLoad(evt) {  document.documentElement.addEventListener('keydown', keyHandler, false); window.addEventListener('DOMMouseScroll', wheel, false);  if (!document)     window.document = evt.target.ownerDocument; }     var xm1, ym1, xm2, ym2;       var isGrabbed = 0;       function Grab(evt)       { 		isGrabbed = 1; 	    	xm1 = evt.clientX; 	    	ym1 = evt.clientY;       };       function Drop(evt)       {          if (isGrabbed = 1)          { 		xm2 = evt.clientX;	    	ym2 = evt.clientY;             var newX = xm2 - xm1;             var newY = ym2 - ym1;            pan(newX, newY);            isGrabbed = 0;         }      }; function pan(dx, dy) {   var ct = document.documentElement.currentTranslate;     ct.x += dx;     ct.y += dy;   control();} function zoom(m) {   var de = document.documentElement;   var oldScale = de.currentScale;   var oldTranslate = { x: de.currentTranslate.x, y: de.currentTranslate.y };   de.currentScale *= m;   var vp_width = window.innerWidth;  var vp_height = window.innerHeight;   de.currentTranslate.x = vp_width/2 -                  (de.currentScale/oldScale) * (vp_width/2 - oldTranslate.x);   de.currentTranslate.y = vp_height/2 -                  (de.currentScale/oldScale) * (vp_height/2 - oldTranslate.y);}function keyHandler(event) {   event.preventDefault();   switch (event.keyCode) {     case 33: /* previous page */       zoom(2);       break;     case 34: /* next page */       zoom(0.5);       break;     case 38: /* up */ 	pan(0,-25);       break;     case 40: /* down */ 	pan(0,25);       break;     case 37: /* left */       pan(-25,0);       break;     case 39: /* right */       pan(25,0);       break;   }   } function handle(delta) {         if (delta < 0) { 		zoom(0.5); 	}         else {		zoom(2);	} } function wheel(event){ 	var delta = 0; 	if (!event) event = window.event; 	if (event.wheelDelta) { 		delta = event.wheelDelta/120; 		if (window.opera) delta = -delta; 	} else if (event.detail) { 		delta = -event.detail/3;	} 	if (delta) 		handle(delta);}    ]]></script> "<< std::endl;
	this->fic << "</svg>" << std::endl;
	this->fic.close();
}

//dessin de segments
void UIfig::addSegFromLeaningPoints(
			const Point<int>& first,
			const Point<int>& last,  
			const Point<int>& Uf, 
			const Point<int>& Ul, 
			const Point<int>& Lf, 
			const Point<int>& Ll, 
			const std::string& couleur) {

	const bool draw_leaning_points = false;

	this->fic << "<g id='DSS' >" << std::endl;

	//definir la boite englobante:
	Point<double> p1, p2, p3, p4;	
	Vecteur<int> slope;
	if (Ul == Uf) 
		slope = Ll - Lf;
	else 
		slope = Ul - Uf;
	if (slope.x == 0) { //pente infinie
		p1 = Point<double>(Ul.x,first.y);
		p2 = Point<double>(Ul.x,last.y);
		p3 = Point<double>(Ll.x,first.y);
		p4 = Point<double>(Ll.x,last.y);
	} else if (slope.y == 0) { //pente nulle
		p1 = Point<double>(first.x,Ul.y);
		p2 = Point<double>(last.x,Ul.y);
		p3 = Point<double>(first.x,Ll.y);
		p4 = Point<double>(last.x,Ll.y);			
	} else {
		double alpha = (double) slope.y/slope.x;
		double invAlpha = -((double) slope.x/slope.y);
		double betaU = Ul.y - alpha*Ul.x;
		double betaL = Ll.y - alpha*Ll.x;
		double x1 = (first.y - invAlpha*first.x - betaU)/(alpha-invAlpha);
		double y1 = alpha*x1 + betaU;	
		double x2 = (last.y - invAlpha*last.x - betaU)/(alpha-invAlpha);
		double y2 = alpha*x2 + betaU;	
		double x3 = (first.y - invAlpha*first.x - betaL)/(alpha-invAlpha);
		double y3 = alpha*x3 + betaL;	
		double x4 = (last.y - invAlpha*last.x - betaL)/(alpha-invAlpha);
		double y4 = alpha*x4 + betaL;	
		p1 = Point<double>(x1,y1);
		p2 = Point<double>(x2,y2);
		p3 = Point<double>(x3,y3);
		p4 = Point<double>(x4,y4);
	}
	//dessiner la boite:
	this->fic << "<polygon points='" << p1.x << "," << p1.y;
	this->fic << " " << p3.x << "," << p3.y;
	this->fic << " " << p4.x << "," << p4.y;
	this->fic << " " << p2.x << "," << p2.y << "' ";
	this->fic << "fill='none' ";
	this->fic << "stroke='" << couleur << "' ";
	this->fic << "stroke-width='0.1'/>" << std::endl;

	if (draw_leaning_points) {
		//dessiner les points d'appui:
		std::vector<Point<int> > v;
		v.push_back(Uf);
		v.push_back(Lf);
		v.push_back(Ul);
		v.push_back(Ll);
		this->addCircularMarks(v,"leaning points","black");
	}

	this->fic << "</g>" << std::endl;


}


